//    SSD1306   AVR. .
#include <avr/io.h>
#define F_CPU 8000000UL
#include <util/delay.h>
#include <stdint.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "ssd1306.h"
#include "image.h"

uint8_t x = 0, y = 0;

#define ORG -10 //  

double flat_0[5][2] =
{
	{50, 15}, //   
	{80, 15}, //   
	{80, 45}, //   
	{50, 45}, //   
	{50, 15}  //   
};

double flat_1[5][2] =
{
	{50+ORG, 15-ORG},
	{80+ORG, 15-ORG},
	{80+ORG, 45-ORG},
	{50+ORG, 45-ORG},
	{50+ORG, 15-ORG}
};

double angle = 5.0 * M_PI / 180;
double midx1, midy1, midx2, midy2;

void rotate (void)
{
	int i;
	for (i = 0; i < 5; i++)
	{
		double xnew, ynew;
		
		xnew = midx1 + (flat_0[i][0] - midx1) * cos (angle) -
		(flat_0[i][1] - midy1) * sin (angle);
		ynew = midy1 + (flat_0[i][0] - midx1) * sin (angle) +
		(flat_0[i][1] - midy1) * cos (angle);
		
		flat_0[i][0] = xnew;
		flat_0[i][1] = ynew;
		
		xnew = midx2 + (flat_1[i][0] - midx2) * cos (angle) -
		(flat_1[i][1] - midy2) * sin (angle);
		ynew = midy2 + (flat_1[i][0] - midx2) * sin (angle) +
		(flat_1[i][1] - midy2) * cos (angle);
		
		flat_1[i][0] = xnew;
		flat_1[i][1] = ynew;
	}

	for (i = 0; i < 4; i++)
	{
		draw_line(flat_0[i][0], flat_0[i][1], flat_0[i+1][0], flat_0[i+1][1]);
		draw_line(flat_1[i][0], flat_1[i][1], flat_1[i+1][0], flat_1[i+1][1]);
		draw_line(flat_0[i][0], flat_0[i][1], flat_1[i][0], flat_1[i][1]);
	}

}

int main(void)
{
	display_init(); //  
	screen_clear(); //  

	midx1 = (flat_0[0][0] + flat_0[1][0]) / 2.0;
	midy1 = (flat_0[1][1] + flat_0[2][1]) / 2.0;
	midx2 = (flat_1[0][0] + flat_1[1][0]) / 2.0;
	midy2 = (flat_1[1][1] + flat_1[2][1]) / 2.0;

	image(0,0,128,64, cat); //  
	screen_update(); //  
	_delay_ms(3000); // 
	screen_clear(); //  

	while(1)
	{
		print_string_5x8(0,1,"0123456789");
		screen_update(); //  
		_delay_ms(3000); // 
		screen_clear(); //  
				
		print_string_8x16(0,1,"0123456789");
	    screen_update(); //  
	    _delay_ms(3000); // 
	    screen_clear(); //  

        for(x = 0; x < 255; x++)
        {
	        print_digit_16x24(0, 0, (x/100)%10);
	        print_digit_16x24(18, 0, (x/10)%10);
	        print_digit_16x24(36, 0, x%10);
			
	        print_digit_16x24(0, 3, ((255 - x)/100)%10);
	        print_digit_16x24(18, 3, ((255 - x)/10)%10);
	        print_digit_16x24(36, 3, (255 - x)%10);
			
	        screen_update();
	        _delay_ms(100);
	        screen_clear();
        }

        for(x = 0; x < 255; x++)
        {
		    print_digit_24x32(52, 0, (255 - x)%10);
		    print_digit_24x32(26, 0, ((255 - x)/10)%10);
		    print_digit_24x32(0, 0, ((255 - x)/100)%10);
		    print_digit_24x32(52, 4, x%10);
		    print_digit_24x32(26, 4, (x/10)%10);
		    print_digit_24x32(0, 4, (x/100)%10);			
	        screen_update();
		    _delay_ms(100);
	        screen_clear();
        }		
				
		//     
        for(uint16_t tmp = 0; tmp < 1024; tmp++)
        {
	        uint8_t x = rand()%128;
	        uint8_t y = rand()%64;
	        draw_fill_rect(x,y,5,5);
	        screen_update();
	        _delay_ms(10);
	        screen_clear();
        }
        //  
		for(x = 0; x < 255; x++)
		{
		rotate();
		screen_update();
		screen_clear();
		}
        //  
		for(x = 0; x < 128;) 
		{
		x = x + 2;
		draw_fill_rect(0,16,x,16);
		screen_update();
		screen_clear();
		}
		for(x = 128; x != 0;)
		{
		x = x - 2;
		draw_fill_rect(0,16,x,16);
		screen_update();
		screen_clear();
		}
        //  
    	for(x = 0; x < 107; x++) 
		{
		draw_fill_rect(x,16,20,16);
		screen_update();
		screen_clear();
		}
		for(x = 107; x != 0; x--)
		{
		draw_fill_rect(x,32,20,16);
		screen_update();
		screen_clear();
		}
		for(x = 0; x < 107; x++)
		{
		draw_fill_rect(x,48,20,16);
		screen_update();
		screen_clear();
		}
		for(x = 107; x != 0; x--)
		{
		draw_fill_rect(x,64,20,16);
		screen_update();
		screen_clear();
		}
        // 
		for(x = 0; x < 31; x++) 
		{
		draw_circle(63,31,x);
		screen_update();
		_delay_ms(50);
		screen_clear();
		}
        // 
		for(x = 0, y = 127; x < 50; x++, y--)
		{
		draw_rect(x,0,y,63);
		screen_update();
		_delay_ms(10);
		screen_clear();
		}
		for(x = 0, y = 63; x < 31; x++, y--)
		{
		draw_rect(50,x,77,y);
		screen_update();
		_delay_ms(10);
		screen_clear();
		}
        // 
		for(x = 0; x < 127; x++)
		{
		y = 0;
		draw_line(63,31,x,y);
		draw_line(63,31,x + 4,y);
		draw_line(63,31,x + 8,y);
		screen_update();
		screen_clear();
		}
		for(y = 0; y < 63; y++)
		{
		x = 127;
		draw_line(63,31,x,y);
		draw_line(63,31,x,y + 4);
		draw_line(63,31,x,y + 8);
		screen_update();
		screen_clear();
		}
		for(x = 127; x != 0; x--)
		{
		y = 63;
		draw_line(63,31,x,y);
		draw_line(63,31,x + 4,y);
		draw_line(63,31,x + 8,y);
		screen_update();
		screen_clear();
		}
		for(y = 63; y != 0; y--)
		{
		x = 0;
		draw_line(63,31,x,y);
		draw_line(63,31,x,y + 4);
		draw_line(63,31,x,y + 8);
		screen_update();
		screen_clear();
		}

		print_string_5x8(0, 0, "RADIOPARTY.RU");
		screen_update();

		start_scroll_right(0, 3);
		_delay_ms(3000);
 		start_scroll_diag_left(7, 7);
 		_delay_ms(3000);
		start_scroll_left(7, 0);
		_delay_ms(3000);
		start_scroll_diag_right(7, 0);
		_delay_ms(3000);
		stop_scroll();
		screen_clear();

	}

}


