#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdint.h>
#include <stdlib.h>

#include "ssd1306.h"
#include "font.h"

uint8_t scr_buffer[1024]; //  

/* Absolute value */
#define ABS(x)   ((x) > 0 ? (x) : -(x))

//   
const uint8_t init[] PROGMEM =
{
0xAE, //  

0xD5, //    
0x80, 
///+-----  0-F/ 0 -   1
//+------  .   0x80

0xA8, //  
0x3F, // 0x3F   , 0x1F - 128x32, 0x3F - 128x64

0xD3, //   (offset)
0x00, //  

0x40, //    0x40 - 0x7F

0x8D, //   
0x14, // 0x10 -  (VCC  ) 0x14 -   DC/DC

0x20, //   
0x00, // 0 -       
      // 1 -       
      // 2 -       

0xA1, //     ( X)
      // A1 -   (/) A0 -  (/)

0xC8, //    
      //    
      // 0 - / (   )
      // 8 - / (   )

0xDA, //   COM
0x12, // 0x02 - 128x32, 0x12 - 128x64

0x81, //   
0x8F, // 0x8F..0xCF

0xD9, //   DC/DC 
0xF1, // 0x22 - VCC  , 0xF1  

0xDB, //   VcomH
0x40, //     0x00..0x70

0xA4, //  

0xA6, // 0xA6 -  , 0xA7 -  

0xAF  //  
};


//   
void display_init(void)
{
SCK_DDR; //    
DAT_DDR;
RST_DDR;
DC_DDR;
CS_DDR;   

SPCR |= (1 << SPE) //  SPI
       |(1 << MSTR); //  ""
//	   |(1 << SPR1); //  SPI fosc/64
SPSR |= (1 << SPI2X); //  SPI fosc/2
SCK_L; //   SCK .
CS_H; //   , 
RST_L; //  
_delay_us(3); //  
RST_H; //    

//       
for(uint8_t i = 0; i < sizeof init; i++) disp_write(0, pgm_read_byte(&init[i]));
}

//   /  
void disp_write(uint8_t mode, uint8_t data) // : 1-, 0-
{
if(mode) DC_H; //  
else DC_L; //  

CS_L; //   , 

SPDR = data; //    
while(!(SPSR & (1<<SPIF))); //   

CS_H; //   , 
}

//    
void screen_clear(void)
{
for(uint16_t i = 0; i < 1024; i++) scr_buffer[i] = 0;
}

//   
void screen_update(void)
{
disp_write(0,0x21); //  
disp_write(0,0);    //  
disp_write(0,127);  //  

disp_write(0,0x22); //  
disp_write(0,0);    //  
disp_write(0,7);    //  
//      
for(uint16_t i = 0; i < 1024; i++) disp_write(1, scr_buffer[i]);
}

//    5x8
void print_char_5x8(uint8_t x, uint8_t y, char sign) // 0..120 / 0..7 / 
{
if(x <= 120 && y < 8)
  {
  if((sign >= 0x20) && (sign <= 0x7F))
  sign -= 32;//      ASCII[0x20-0x7F]
  else if (sign >= 0xC0)
  sign -= 96; //      CP1251[0xC0-0xFF]
  else  sign = 85; //   (       )
  
  for(uint8_t i = 0; i < 5; i++)
	{
	scr_buffer[128*y + x + i] = pgm_read_byte(&font_5x8[5*sign + i]);
	}
  }
}

//    5x8
void print_string_5x8(uint8_t x, uint8_t y, char *str) // 0..120 / 0..7 / 
{
while(*str != '\0')
	{
	print_char_5x8(x, y, *str++);
	x+=6;
	}
}

//    8x16
void print_char_8x16(uint8_t x, uint8_t y, uint8_t sign) // 0..120 / 0..7 / 
{
	uint8_t i;
	if(x <= 120 && y < 8)
	{
		if((sign >= 0x20) && (sign <= 0x7F))
		sign -= 32;//      ASCII[0x20-0x7F]
		else if ( sign >= 0xC0 )
		sign -= 96; //      CP1251[0xC0-0xFF]
		else  sign = 85; //   (       )
		for(i = 0; i < 8; i++)
		{
			scr_buffer[128*y + x + i - 128] = pgm_read_byte(&font_8x16[sign * 16 + i]);
		}
		for(i = 0; i < 8; i++)
		{
			scr_buffer[128*y + x + i] = pgm_read_byte(&font_8x16[sign * 16 + i + 8]);
		}
	}
}
//    8x16
void print_string_8x16(uint8_t x, uint8_t y, char *str) // 0..120 / 0..7 / 
{
while(*str != '\0')
	{
		print_char_8x16(x, y, *str++);
		x+=9;
	}
}

//    16x24
void print_digit_16x24(uint8_t x, uint8_t y, uint8_t sign) // 0..120 / 0..7 / 
{
	uint8_t i;
	if(x <= 120 && y < 8)
	{
		for(i = 0; i < 16; i++)
		{
			scr_buffer[128*y + x + i] = pgm_read_byte(&font_16x24[sign * 48 + i]);
		}
		for(i = 0; i < 16; i++)
		{
			scr_buffer[128*y + x + i + 128] = pgm_read_byte(&font_16x24[sign * 48 + i + 16]);
		}
		for(i = 0; i < 16; i++)
		{
			scr_buffer[128*y + x + i + 256] = pgm_read_byte(&font_16x24[sign * 48 + i + 32]);
		}
	}
}

//    24x32
void print_digit_24x32(uint8_t x, uint8_t y, uint8_t sign) // 0..120 / 0..7 / 
{
	uint8_t i = 0;
	if(x <= 120 && y < 8)
	{
		for(i = 0; i < 24; i++)
		{
			scr_buffer[128*y + x + i] = pgm_read_byte(&font_24x32[sign * 96 + i]);
		}
		for(i = 0; i < 24; i++)
		{
			scr_buffer[128*y + x + i + 128] = pgm_read_byte(&font_24x32[sign * 96 + i + 24]);
		}
		for(i = 0; i < 24; i++)
		{
			scr_buffer[128*y + x + i + 256] = pgm_read_byte(&font_24x32[sign * 96 + i + 48]);
		}
		for(i = 0; i < 24; i++)
		{
			scr_buffer[128*y + x + i + 384] = pgm_read_byte(&font_24x32[sign * 96 + i + 72]);
		}
	}
}

//   (1px)
void draw_pixel(uint8_t x, uint8_t y) //0..127 / 0..63
{
if(x <= 127 && y <= 63) SET_BIT(scr_buffer[x + 128*(y/8)], y%8);
}

//    
void draw_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2)
{
int16_t P;
int8_t x, y, addx, addy, dx, dy;

x = x1;
y = y1;

dx = ABS((int8_t)(x2-x1));
dy = ABS((int8_t)(y2-y1));

if(x1>x2) addx = -1;
else addx = 1;

if(y1>y2) addy = -1;
else addy = 1;

if(dx>=dy)
	{
	P = 2*dy-dx;

	for(uint8_t i=0; i<=dx; ++i)
		{
		draw_pixel(x, y);

		if(P<0)
			{
			P += 2*dy;
			x += addx;
			}
		else
			{
			P += 2*dy-2*dx;
			x += addx;
			y += addy;
			}
		}
	}
else
	{
	P = 2*dx-dy;

	for(uint8_t i=0; i<=dy; ++i)
		{
		draw_pixel(x, y);

		if(P<0)
			{
			P += 2*dx;
			y += addy;
			}
		else
			{
			P += 2*dx-2*dy;
			x += addx;
			y += addy;
			}
		}
	}
}

//   
// x1 -   x   
// y1 -   y   
// x2 -   x   
// y2 -   y   
void draw_rect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2)
{
uint8_t i;

if((x2 > x1) && (y2 > y1))
{
//   
  for(i = x1; i <= x2; i++)
    {
    draw_pixel(i, y1);
    draw_pixel(i, y2);
    }
//   
  for(i = y1; i <= y2; i++)
    {
    draw_pixel(x1, i);
    draw_pixel(x2, i);
    }
}
}

//    
// x -   x (  )
// y -   y (  )
// height ->  ( )
// width  ->  ( )
void draw_fill_rect(uint8_t x, uint8_t y, uint8_t width, uint8_t height)
{
uint8_t tmpIdxX, tmpIdxY, tmp;

if(height > y) tmp = 0;
else tmp = y - height + 1;

//  
for(tmpIdxY = tmp; tmpIdxY <= y; tmpIdxY++)
{
  for(tmpIdxX = x; tmpIdxX < (x + width); tmpIdxX++)
     {
     draw_pixel(tmpIdxX, tmpIdxY);
     }
}
}

//   
void draw_triangle(uint8_t x1, uint8_t y1,uint8_t x2, uint8_t y2, uint8_t x3, uint8_t y3)
{
draw_line(x1, y1, x2, y2);
draw_line(x2, y2, x3, y3);
draw_line(x3, y3, x1, y1);
}

//    
void draw_fill_triangle(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t x3, uint8_t y3) 
{
	int16_t deltax = 0, deltay = 0, x = 0, y = 0, xinc1 = 0, xinc2 = 0, 
	yinc1 = 0, yinc2 = 0, den = 0, num = 0, numadd = 0, numpixels = 0, 
	curpixel = 0;
	
	deltax = ABS(x2 - x1);
	deltay = ABS(y2 - y1);
	x = x1;
	y = y1;

	if (x2 >= x1) {
		xinc1 = 1;
		xinc2 = 1;
	} else {
		xinc1 = -1;
		xinc2 = -1;
	}

	if (y2 >= y1) {
		yinc1 = 1;
		yinc2 = 1;
	} else {
		yinc1 = -1;
		yinc2 = -1;
	}

	if (deltax >= deltay){
		xinc1 = 0;
		yinc2 = 0;
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
		numpixels = deltax;
	} else {
		xinc2 = 0;
		yinc1 = 0;
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
		numpixels = deltay;
	}

	for (curpixel = 0; curpixel <= numpixels; curpixel++) {
        draw_line(x, y, x3, y3);

		num += numadd;
		if (num >= den) {
			num -= den;
			x += xinc1;
			y += yinc1;
		}
		x += xinc2;
		y += yinc2;
	}
}

//   
// x, y -   
// radius -  
void draw_circle(uint8_t x, uint8_t y, uint8_t radius)
{
signed char xc = 0;
signed char yc = 0;
signed char p = 0;

yc = radius;
p = 3 - (radius<<1);
while (xc <= yc)  
{
draw_pixel(x + xc, y + yc);
draw_pixel(x + xc, y - yc);
draw_pixel(x - xc, y + yc);
draw_pixel(x - xc, y - yc);
draw_pixel(x + yc, y + xc);
draw_pixel(x + yc, y - xc);
draw_pixel(x - yc, y + xc);
draw_pixel(x - yc, y - xc);
if(p < 0) p += (xc++ << 2) + 6;
else p += ((xc++ - yc--)<<2) + 10;
}
}

//    
// x, y -   
// radius -  
void draw_fill_circle(uint8_t x, uint8_t y, uint8_t radius) 
{
int16_t f = 1 - radius;
int16_t ddF_x = 1;
int16_t ddF_y = -2 * radius;
int16_t xc = 0;
int16_t yc = radius;

draw_pixel(x, y + radius);
draw_pixel(x, y - radius);
draw_pixel(x + radius, y);
draw_pixel(x - radius, y);
draw_line(x - radius, y, x + radius, y);

while(xc < yc)
{
  if (f >= 0) 
  {
  yc--;
  ddF_y += 2;
  f += ddF_y;
  }
xc++;
ddF_x += 2;
f += ddF_x;

draw_line(x - xc, y + yc, x + xc, y + yc);
draw_line(x + xc, y - yc, x - xc, y - yc);

draw_line(x + yc, y + xc, x - yc, y + xc);
draw_line(x + yc, y - xc, x - yc, y - xc);
}
}


//   .
// 0..120 / 0..7 / width -  / height -  / image
void image(uint8_t x, uint8_t y, uint8_t width, uint8_t height, const uint8_t bitmap[])
{
uint8_t a,b;
uint16_t baitch = 0;
height = height/8;

for(b = 0; b < height; b++)
{
  for (a = 0; a < width; a++)
    {
    scr_buffer[128*x + y + a + (b*128)] = pgm_read_byte(&bitmap[baitch]);
	baitch++;
	}
}
}

//    
void start_scroll_right(uint8_t start, uint8_t stop)
{
disp_write(0, 0x26);  //    
disp_write(0, 0x00);
disp_write(0, start); //  
disp_write(0, 0x00);  // 
disp_write(0, stop);  //  
disp_write(0, 0x00);
disp_write(0, 0xFF);
disp_write(0, 0x2F);  //  
}

//    
void start_scroll_left(uint8_t start, uint8_t stop)
{
disp_write(0, 0x27);  //    
disp_write(0, 0x00);
disp_write(0, start); //  
disp_write(0, 0x00);  // 
disp_write(0, stop);  //  
disp_write(0, 0x00);
disp_write(0, 0xFF);
disp_write(0, 0x2F);  //  
}

//      
void start_scroll_diag_right(uint8_t start, uint8_t stop)
{
disp_write(0, 0xA3);  //    
disp_write(0, 0x00);  //      
disp_write(0, 64);    //     
disp_write(0, 0x29);  //  .  .  
disp_write(0, 0x00);
disp_write(0, start); //  
disp_write(0, 0x00);  // 
disp_write(0, stop);  //  
disp_write(0, 0x01);  // , 0x01 - , 0x3F - 
disp_write(0, 0x2F);  //  
}

//      
void start_scroll_diag_left(uint8_t start, uint8_t stop)
{
disp_write(0, 0xA3);  //    
disp_write(0, 0x00);  //      
disp_write(0, 64);    //     
disp_write(0, 0x2A);  //  .  .  
disp_write(0, 0x00);
disp_write(0, start); //  
disp_write(0, 0x00);  // 
disp_write(0, stop);  //  
disp_write(0, 0x3F);  // , 0x01 - , 0x3F -  
disp_write(0, 0x2F);  //  
}

void stop_scroll(void)
{
disp_write(0, 0x2E);
}





