#ifndef __SSD1306_H
#define __SSD1306_H

#include <avr/io.h>

#define SCK_PORT PORTB // SCK(D0)
#define SCK_PIN  5

#define DAT_PORT PORTB // SDA(D1)
#define DAT_PIN  3

#define RST_PORT PORTB // RES
#define RST_PIN  0

#define DC_PORT  PORTB // DC
#define DC_PIN   1

#define CS_PORT  PORTB // CS
#define CS_PIN   2

#define SET_BIT(reg, bit)  reg |= (1<<bit)
#define CLR_BIT(reg, bit)  reg &= (~(1<<bit))
#define INV_BIT(reg, bit)  reg ^= (1<<bit)

#define DDRX(x) (*(&x-1))

#define SCK_DDR  SET_BIT(DDRX(SCK_PORT), SCK_PIN)
#define DAT_DDR  SET_BIT(DDRX(DAT_PORT), DAT_PIN)
#define RST_DDR  SET_BIT(DDRX(RST_PORT), RST_PIN)
#define DC_DDR   SET_BIT(DDRX(DC_PORT), DC_PIN)
#define CS_DDR   SET_BIT(DDRX(CS_PORT), CS_PIN)


#define SCK_H  SET_BIT(SCK_PORT, SCK_PIN)
#define SCK_L  CLR_BIT(SCK_PORT, SCK_PIN)

#define DAT_H  SET_BIT(DAT_PORT, DAT_PIN)
#define DAT_L  CLR_BIT(DAT_PORT, DAT_PIN)

#define RST_H  SET_BIT(RST_PORT, RST_PIN)
#define RST_L  CLR_BIT(RST_PORT, RST_PIN)

#define DC_H   SET_BIT(DC_PORT, DC_PIN)
#define DC_L   CLR_BIT(DC_PORT, DC_PIN)

#define CS_H   SET_BIT(CS_PORT, CS_PIN)
#define CS_L   CLR_BIT(CS_PORT, CS_PIN)

void display_init(void); 
void disp_write(uint8_t mode, uint8_t data); 
void screen_clear(void);
void screen_update(void);
void print_char_5x8(uint8_t x, uint8_t y, char sign);
void print_string_5x8(uint8_t x, uint8_t y, char *str);
void print_char_8x16(uint8_t x, uint8_t y, uint8_t sign);
void print_string_8x16(uint8_t x, uint8_t y, char *str);
void print_digit_16x24(uint8_t x, uint8_t y, uint8_t sign);
void print_digit_24x32(uint8_t x, uint8_t y, uint8_t sign);
void draw_pixel(uint8_t x, uint8_t y);
void draw_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void draw_rect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void draw_fill_rect(uint8_t x, uint8_t y, uint8_t width, uint8_t height);
void draw_circle(uint8_t x, uint8_t y, uint8_t radius);
void image(uint8_t x, uint8_t y, uint8_t width, uint8_t height, const uint8_t bitmap[]);
void start_scroll_right(uint8_t start, uint8_t stop);
void start_scroll_left(uint8_t start, uint8_t stop);
void start_scroll_diag_right(uint8_t start, uint8_t stop);
void start_scroll_diag_left(uint8_t start, uint8_t stop);
void stop_scroll(void);

#endif

